import { world, system, EntityEquippableComponent, EquipmentSlot, ItemStack, EnchantmentType, EffectType, EntityHealthComponent, Player, EntityComponentTypes } from '@minecraft/server';
import { items } from './itemRegister';
const cooldowns = new Map();
const cooldownPercentages = new Map();
const lastSelectedItem = new Map();
const activeCooldowns = new Set();
let lastMainHandItem = null;
const unicodes = ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', ''];
const intervalIds = new Map();
let intervalRunning = true;
const vec2YSub = (a, b) => {return a.y - b.y}
const recentHits = new Map();

world.afterEvents.entityHurt.subscribe((event) => {
    const entity = event.hurtEntity;
    if (entity?.typeId === "minecraft:player") {
        world.playSound("hit", entity.location)
		if (entity.getEffect("minecraft:poison") || entity.getEffect("minecraft:wither") || entity.getComponent(EntityComponentTypes.OnFire)) {
			entity.applyKnockback(0,0,0,0);
		}
    }
});

export let playerdata = {};
system.runInterval(() => {
  for (const player of world.getPlayers()) {
    const isSprinting = player.isSprinting;
	const isFalling = player.isFalling;
    const isOnGround = player.isOnGround;
    system.runTimeout(() => {
      playerdata[player.id] = {
        isSprinting,
		isFalling,
        isOnGround,
      };
	  if (world.getDynamicProperty("java") === true) {
	  if (player.getDynamicProperty("kb") === undefined) {
			player.setDynamicProperty("kb", false);
		}
		if (player.getDynamicProperty("kbhit") === undefined) {
			player.setDynamicProperty("kbhit", false);
		}
	  if (!isSprinting && player.getDynamicProperty("kbhit") === true) {
			player.setDynamicProperty("kb", false);
			player.setDynamicProperty("kbhit", false);
	  }
	  if (isSprinting && player.getDynamicProperty("kbhit") === false) {
		  player.setDynamicProperty("kbhit", true);
		  player.setDynamicProperty("kb", true);
	  }
	  }
    }, 1);
  }
});

world.afterEvents.projectileHitEntity.subscribe(evd => {
  const { source: attacker, projectile } = evd;
  const victim = evd.getEntityHit().entity;
  if (!attacker) {return}

  if (projectile?.typeId === "minecraft:arrow") {
    attacker.runCommand("playsound random.orb @s");
  }
});



function startCooldown(player, cooldownType, cooldownDuration) {
    const cooldownStart = Date.now();
    const cooldownEnd = cooldownStart + cooldownDuration * 1000;
    const playerCooldowns = cooldowns.get(player.id) || {};
    playerCooldowns[cooldownType] = { cooldownStart, cooldownEnd };
    cooldowns.set(player.id, playerCooldowns);
    activeCooldowns.add(player.id);
    player.addTag("usingItem");
    system.runTimeout(() => {
        player.removeTag("usingItem");
    }, 20);
}

function getCooldownPercent(player, cooldownType) {
    const playerCooldowns = cooldowns.get(player.id) || {};
    const cooldownData = playerCooldowns[cooldownType];
    if (cooldownData) {
        const remainingTime = Math.max(cooldownData.cooldownEnd - Date.now(), 0);
        const cooldownPercent = Math.max(100 - (remainingTime / (cooldownData.cooldownEnd - cooldownData.cooldownStart)) * 100, 0);
        return cooldownPercent;
    }
    return 100;
}




function getPlayerCooldownPercent(playerId) {
  return cooldownPercentages.get(playerId) || 100;}
  system.runInterval(() => {
    activeCooldowns.forEach((playerId) => {
        const player = world.getAllPlayers().find(p => p.id === playerId);
        if (!player) {
            activeCooldowns.delete(playerId);
            return;
        }
        const cooldownPercent = getCooldownPercent(player, "sweep");
        const lastPercent = player.getDynamicProperty("lastCooldownPercent") || 0;
        if (Math.floor(cooldownPercent) !== Math.floor(lastPercent)) {
            player.onScreenDisplay.setTitle(`_sweepnslash:crs:f:${cooldownPercent.toFixed(0)}%`, {
                fadeInDuration: 0,
                fadeOutDuration: 0,
                stayDuration: 0,
            });
            player.setDynamicProperty("lastCooldownPercent", cooldownPercent);
        }
        if (cooldownPercent >= 100) {
			player.onScreenDisplay.setTitle(`_sweepnslash:non`, {
                fadeInDuration: 0,
                fadeOutDuration: 0,
                stayDuration: 0,
            });
			
            activeCooldowns.delete(playerId);
        }
    });
}, 1);



function activateCooldown(player) {
  const inventory = player.getComponent('minecraft:inventory')?.container;
  const selectedItem = inventory?.getItem(player.selectedSlotIndex);
  if (selectedItem) {
    let cooldownApplied = false;
    for (const item of items) {
		if (player.hasTag("sword")) {
			return;
		  }
     if (selectedItem && selectedItem.typeId.includes("mace")) {
		handleCooldowns(player, selectedItem, 2.2, 'sweep'); 
    return;}
      if (selectedItem.typeId.includes(item.item)) {
        cooldownApplied = true;
        for (const specificItem of item.specific) {
          if (
            specificItem.type === 'all' || 
            (Array.isArray(specificItem.type) && specificItem.type.some(type => selectedItem.typeId.includes(type)))
          ) {
  
            handleCooldowns(player, selectedItem, specificItem.duration, 'sweep');
          }
        }
        break;
      }
    }
    if (!cooldownApplied) {
      if (selectedItem.hasTag('is_sword')) {
        handleCooldowns(player, selectedItem, 0.5, 'sweep');
      } else if (selectedItem.hasTag('is_axe')) {
        handleCooldowns(player, selectedItem, 1, 'sweep');
      } else {
        handleCooldowns(player, selectedItem, 0.4, 'sweep');
      }
    }
  } else {
    handleCooldowns(player, null, 0.4, 'sweep');
  }
}


function handleCooldowns(player, item, cooldownDuration, cooldownType) {
    const cooldownKey = `cd:${cooldownType}Timer`;
    const hasteSpeed = getHasteSpeed(player);
    const miningSpeed = getMiningSpeed(player);
    const adjustedCooldownDuration = cooldownDuration * (1 - (hasteSpeed * 0.5) + (miningSpeed * 0.5));
    startCooldown(player, cooldownType, adjustedCooldownDuration);
}

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        if (!player) return;
        const inv = player.getComponent("inventory")?.container;
        const item = inv?.getItem(player.selectedSlotIndex);
        const lastItem = player.getDynamicProperty("last_item_held") || "";

        if (item) {
            if (item.typeId === "minecraft:mace" && (lastItem.includes("sword") || lastItem.includes("_axe"))) {
              system.runTimeout(() => {
                activateCooldown(player);
              }, 5);
            }
            if (item.typeId !== lastItem) {
                activateCooldown(player);
                player.setDynamicProperty("last_item_held", item.typeId);
            }
        } else {
            player.setDynamicProperty("last_item_held", "");
        }
    });
});

system.runInterval(function cartExplode() {
    world.getAllPlayers().forEach((player) => {
        const arrows = player.dimension.getEntities({ type: 'arrow' })
        for (const arrow of arrows) {
            const arrowfire = arrow.getComponent(EntityComponentTypes.OnFire)
            if (arrowfire) {
                arrow.runCommandAsync('event entity @e[type=tnt_minecart,x=~-4,y=~0.3,z=~-4,dx=6,dy=5,dz=5,c=1] minecraft:on_instant_prime')
            }
        }
    })
})

function spawnCriticalHitParticles(dimension, location) {
  const particlePos = { x: location.x, y: location.y + 0.625, z: location.z };
  for (let i = 0; i < 3; i++) {
    dimension.spawnParticle("dave:critical_hit_custom", particlePos);
  }
}

world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemUsed = event.itemStack;
	 if (player && itemUsed) {
        player.addTag("usingItem");
    }
});

world.afterEvents.itemStopUse.subscribe(event => {
    const player = event.source;
    if (player) {
        player.removeTag("usingItem");
    }
});

function isBlockingWithShield(player) {
  if (player.typeId !== "minecraft:player") return false;
  const inventory = player.getComponent("minecraft:inventory")?.container;
  const mainHandItem = inventory?.getItem(player.selectedSlotIndex);
  const offhandItem = player.getComponent("minecraft:equippable")?.getEquipment(EquipmentSlot.Offhand);
  return player.isSneaking && (
      (mainHandItem && mainHandItem?.typeId === "minecraft:shield") || 
      (offhandItem && offhandItem?.typeId === "minecraft:shield")
  );
}

function getHasteSpeed(entity) {
    const hasteEffect = entity.getEffect("minecraft:haste");
    if (!hasteEffect) {return 0;}
    const amplifier = hasteEffect.amplifier;
    const hasteSpeed = 0.03 * (amplifier + 1);
    return hasteSpeed;
}

function getMiningSpeed(entity) {
    const miningEffect = entity.getEffect("minecraft:mining_fatigue");
    if (!miningEffect) {return 0;}
    const amplifier = miningEffect.amplifier;
    const miningSpeed = 0.03 * (amplifier + 1);
    return miningSpeed;
}

world.afterEvents.entityHitEntity.subscribe(async (event) => {
    const attacker = event.damagingEntity;
    const target = event.hitEntity;
    const attackerViewDirection = attacker.getViewDirection();
	  const PRot = attacker.getRotation();
    const DRot = target.getRotation();
    const YDeferent = vec2YSub(PRot, DRot);
    if (!attacker || !target) {return;}
    if (target.typeId == "minecraft:boat") {
      target.applyDamage(100, {
          cause: "none",
          source: attacker,
      });
      return;
    }
    
    const inventory = attacker.getComponent("minecraft:inventory")?.container;
    if (isBlockingWithShield(target) && !(YDeferent <= 90 && YDeferent >= -90) && !target.hasTag("usingItem") && !target.hasTag("shield_b")) { 
        attacker.applyKnockback(0,0,0,0);
        if (!inventory) return;
        const mainHandItem = inventory.getItem(attacker.selectedSlotIndex);
        if (mainHandItem && mainHandItem.typeId.includes("axe")) {
            target.addTag("shield_b");
            system.runTimeout(() => {
                target.removeTag("shield_b");
                target.runCommandAsync("playsound random.orb @s");
            }, 100);
        }
      return;
    }
      if (attacker && target) {
          recentHits.set(target.id, attacker);
          system.runTimeout(() => {
            if (attacker) {
              recentHits.delete(target.id, attacker);
            }
          }, 20);
      }
	  handleHit(attacker, target);
	  activateCooldown(attacker);
});


system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { id, sourceEntity } = event;
	const cooldownPercent = getCooldownPercent(sourceEntity, "sweep");
    if (id === "bf:attack" && cooldownPercent > 50) {
        if (sourceEntity?.typeId === "minecraft:player") {
            activateCooldown(sourceEntity);
        }
    }
});

function isWearingNetheriteArmor(player) {
  const equippableComponent = player.getComponent("equippable");
  if (!equippableComponent) {
      return false;
  }
  const armorSlots = {
      "Head": "minecraft:netherite_helmet",
      "Chest": "minecraft:netherite_chestplate",
      "Legs": "minecraft:netherite_leggings",
      "Feet": "minecraft:netherite_boots"
  };
  let count = 0;
  for (const [slot, expectedItem] of Object.entries(armorSlots)) {
      const equippedItem = equippableComponent.getEquipment(slot)?.typeId;

      if (equippedItem === expectedItem) {
          count++;
      }
  }
  return count >= 3;
}


function getStrengthDamage(entity) {
  const strengthEffect = entity.getEffect("minecraft:strength");
  if (!strengthEffect) return 0;
  const amplifier = strengthEffect.amplifier;
  const strengthDamage = 3 * (amplifier + 1);
  return strengthDamage;
}

function getWeaknessDamage(entity) {
  const weaknessEffect = entity.getEffect("minecraft:weakness");
  if (!weaknessEffect) {return 0;}
  const amplifier = weaknessEffect.amplifier;
  const weaknessDamage = 3 * (amplifier + 1);
  return weaknessDamage;
}

function getSharpnessDamage(item) {
  const enchantableComponent = item.getComponent("enchantable");
  if (!enchantableComponent) {return 0;}
  const sharpnessLevel = enchantableComponent.getEnchantment(new EnchantmentType("sharpness"))?.level || 0;
  const sharpnessDamageMap = {1: 1, 2: 1.5, 3: 2, 4: 2.5, 5: 3};
  return sharpnessDamageMap[sharpnessLevel] || 0;
}

function handleHit(attacker, hitEntity) {
  const attackerViewDirection = attacker.getViewDirection();
  const cooldownPercent = getCooldownPercent(attacker, "sweep");
  if (cooldownPercent < 80) {
	hitEntity.applyKnockback(0,0,0,0);
	return;
  } else {
    if (isWearingNetheriteArmor(hitEntity)) {
      hitEntity.applyKnockback(
        -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
        Math.cos(attacker.getRotation().y * (Math.PI / 180)),
        0.5,
        0.20
      );
    } else {
      hitEntity.applyKnockback(
        -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
        Math.cos(attacker.getRotation().y * (Math.PI / 180)),
        0.3,
        0.28
      );
    }
	const hitLocation = hitEntity.location;
		const dimension = hitEntity.dimension;
		const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 1,
				z: hitLocation.z,
			};
	dimension.spawnParticle("bf:damage_indicator_emitter", particlePos);
    world.playSound("hit", hitEntity.location);
  }
  let critvolplus = 1;
  let critvolminus = 1;
  if (cooldownPercent >= 80 && cooldownPercent <= 100) {
    const t = (cooldownPercent - 75) / 25;
    critvolminus = 1.4 - 0.4 * t;
    critvolplus = 1.4 + 0.4 * t;
  }
	const inventory = attacker.getComponent("minecraft:inventory")?.container;
	const selectedItem = inventory?.getItem(attacker.selectedSlotIndex);
	if (!selectedItem) {return;}
	let isCritical = ((!playerdata[attacker.id]?.isOnGround && playerdata[attacker.id]?.isFalling && !attacker.isInWater) || (attacker.isInWater && !playerdata[attacker.id]?.isSprinting)) && cooldownPercent > 80;
	let isKnockBack = (playerdata[attacker.id]?.isSprinting || selectedItem.typeId.includes("axe")) && (!playerdata[attacker.id]?.isFalling || playerdata[attacker.id]?.isOnGround) && cooldownPercent > 80;
	if (world.getDynamicProperty("java") === true) {
		isCritical = !playerdata[attacker.id]?.isOnGround && playerdata[attacker.id]?.isFalling && !attacker.isInWater && attacker.getDynamicProperty("kb") === false && cooldownPercent > 80;
		isKnockBack = (playerdata[attacker.id]?.isSprinting || selectedItem.typeId.includes("axe")) && cooldownPercent > 55 && attacker.getDynamicProperty("kb") === true;
	}
	const sharpness = getSharpnessDamage(selectedItem)
	if (cooldownPercent > 80 && sharpness > 0) {
		const hitLocation = hitEntity.location;
		const dimension = hitEntity.dimension;
		const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 1,
				z: hitLocation.z,
			};
		dimension.spawnParticle("bf:enchanted_hit_emitter", particlePos);
	}
	if (isKnockBack) {
    if (hitEntity.isFalling) {
      if (isWearingNetheriteArmor(hitEntity)) {
        hitEntity.applyKnockback(
          -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
          Math.cos(attacker.getRotation().y * (Math.PI / 180)),
          1.2,
          -0.4
        );
      } else {
        hitEntity.applyKnockback(
          -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
          Math.cos(attacker.getRotation().y * (Math.PI / 180)),
          0.9,
          -0.5
        );
      }
    } else {
      if (isWearingNetheriteArmor(hitEntity)) {
        hitEntity.applyKnockback(
          -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
          Math.cos(attacker.getRotation().y * (Math.PI / 180)),
          1.2,
          0.30
        );
      } else {
        hitEntity.applyKnockback(
          -Math.sin(attacker.getRotation().y * (Math.PI / 180)),
          Math.cos(attacker.getRotation().y * (Math.PI / 180)),
          0.9,
          0.38
        );
      }
    }
		if (world.getDynamicProperty("java") === true) {
		attacker.setDynamicProperty("kb", false);
		}
		world.playSound("knockback", hitEntity.location);
	} else if (isCritical) {
		const hitLocation = hitEntity.location;
		if (hitLocation) {
			const dimension = hitEntity.dimension;
			if (selectedItem.typeId.includes("_axe")) {critvolminus = critvolminus * 0.9; critvolplus = critvolplus * 2;}
      if (selectedItem.typeId.includes("mace")) {critvolminus = critvolminus * 1; critvolplus = critvolplus * 0;}
			world.playSound("critical_hit", hitLocation, {
				pitch: critvolminus,
				volume: critvolplus,
			});

			const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 2,
				z: hitLocation.z,
			};
			dimension.spawnParticle("bf:critical_hit_emitter", particlePos);
		}
	} else if (selectedItem.typeId.includes("sword") && cooldownPercent >= 80) {
		const attackDimension = world.getDimension(attacker.dimension.id);
		const attackDirection = attacker.getRotation().y * (Math.PI / 180);
		const offsetX = -Math.sin(attackDirection) * 2;
		const offsetZ = Math.cos(attackDirection) * 2;
		const particlePosition = {
		  x: attacker.location.x + offsetX,
		  y: attacker.location.y + 0.2,
		  z: attacker.location.z + offsetZ
		};
		attackDimension.spawnParticle("dave:sweep", particlePosition);
		world.playSound("sweep", particlePosition);
		attackDimension.getEntities({
		  location: hitEntity.location,
		  maxDistance: 1.5
		}).forEach(entity => {
		  if (entity.id === attacker.id || entity.id === hitEntity.id) return;

		  const damage = 7 * (Math.random() * 0.25 + 0.5);
		  entity.applyDamage(damage, {
			cause: "none",
			damagingEntity: attacker
		  });
		});
	}
  let caldamage = 1;
  if (selectedItem) {
    const itemId = selectedItem.typeId;
    const matchedItem = items.find((item) => item.typeId === itemId);
    if (matchedItem) {
      const { extraDamage: damage, cooldown } = matchedItem;
      caldamage =
        damage +
        getSharpnessDamage(selectedItem) +
        getStrengthDamage(attacker) -
        getWeaknessDamage(attacker);
      const thresholds = [
        0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60,
        65, 70, 75, 80, 85, 90, 95, 100,
      ];

      const outputs = [
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3125, 0.3445,
        0.3805, 0.4205, 0.4645, 0.5125, 0.5645, 0.6205, 0.6805, 0.7445, 0.8125,
        0.8845, 0.9605, 1.0,
      ];
      for (let i = 0; i < thresholds.length; i++) {
        if (cooldownPercent <= thresholds[i]) {
          caldamage *= outputs[i];
          break;
        }
      }

      if (isCritical) {
        caldamage *= 1.5;
      }
	  if (attacker.hasTag("sword")) {
	  caldamage = caldamage + 10
	  }
    let saturationTime = hitEntity.getDynamicProperty("saturationTime") || 0;
    if (saturationTime > 0) {
      caldamage = caldamage * 0.8 
    }

     if (!hitEntity.hasTag("hit")) {
		hitEntity.applyDamage(caldamage, {
        cause: "none",
        source: attacker,
      });
	  hitEntity.addTag("hit");
	  system.runTimeout(() => {
                hitEntity.removeTag("hit");        
      }, 10);
	  }
    }
  }
}

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    player.removeTag("hit");     
  }
}, 100);

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    player.removeTag("fire");
  }
}, 1000);


system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (player.getComponent(EntityComponentTypes.OnFire) && !player.hasTag("fire")) {
      player.addTag("fire");
      system.runTimeout(() => {
        player.removeTag("fire");
        player.extinguishFire(true);
      }, 80);
    }
  }
}, 1);

system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const playerPos = player.location;
        const dimension = player.dimension; // Get player's dimension

        // Get all entities in the player's dimension
        const entities = dimension.getEntities();

        // Filter dolphins within the detection radius
        const nearbyDolphins = entities.filter(entity => 
            entity.typeId === "minecraft:dolphin" &&
            entity.location &&
            Math.hypot(
                entity.location.x - playerPos.x,
                entity.location.y - playerPos.y,
                entity.location.z - playerPos.z
            ) <= 10
        );

        if (nearbyDolphins.length > 0) {
            player.runCommandAsync(`event entity @s bf:swim_add`);
        } else {
            system.runTimeout(() => {
              if (player && player.isValid()) {
                player.runCommandAsync(`event entity @s bf:swim_remove`); 
              }  
		  }, 100);
        }
    }
}, 10);

const itemList = [
  "bf:wooden_axe",
  "bf:wooden_sword",
  "bf:stone_axe",
  "bf:stone_sword",
  "bf:iron_axe",
  "bf:iron_sword",
  "bf:golden_axe",
  "bf:golden_sword",
  "bf:diamond_axe",
  "bf:diamond_sword",
  "bf:netherite_axe",
  "bf:netherite_sword"
];

function itemDurabilityDecrease(itemStack, player, decreaseAmount) {
  const durability = itemStack.getComponent("durability");
  if (durability) {
    durability.damage += decreaseAmount;
    if (durability.damage >= durability.maxDurability) {
      const inv = player.getComponent("inventory").container;
      inv.setItem(player.selectedSlotIndex, undefined);
      world.playSound("random.break", player.location);
      return null;
    }
  }
  return itemStack;
}

world.afterEvents.playerBreakBlock.subscribe((eventData) => {
  const { player, itemStackBeforeBreak } = eventData;
    if (!itemStackBeforeBreak) {return}
  if (itemList.includes(itemStackBeforeBreak.typeId)) {
    const newItem = itemDurabilityDecrease(itemStackBeforeBreak, player, 1);
    const inv = player.getComponent("inventory").container;
    if (newItem) {
      inv.setItem(player.selectedSlotIndex, newItem);
    }
  }
});


world.afterEvents.entityDie.subscribe((event) => {
    const { deadEntity } = event;
    if (world.getDynamicProperty("death") === true) {
      if (deadEntity.typeId === "minecraft:player") {
          const killer = recentHits.get(deadEntity.id);
          if (killer && killer.typeId === "minecraft:player") {
              const deathMessage = `§l${deadEntity.name} was slain by ${killer.name}`;
              world.sendMessage(deathMessage);
          }
          recentHits.delete(deadEntity.id);
      }
    }
});