/**
 * How to add custom items:
 *   {
 *       item: "item_name",
 *     specific: [
 *          {
 *              type: "all", // dont need to mess with this
 *              duration: 0.6 // attack speed
 *         }
 *     ],
 * },
 *	add this in the array below if your having trouble ask me i can help 
 *  discord: lavazoid123
 *
 * Documentation for itemRegister.js
 *
 * This file contains the definition and registration of various items in the form of an array of objects.
 * Each object within the array represents a different item and includes specific properties related to that item.
 *
 * Structure of Data:
 * - itemRegister: An array containing objects with the following properties:
 *   - item: The name of the item as a string.
 *   - specific: An array of objects that define specific characteristics of the item, such as type and duration.
 * 
 * Details of the 'item' Property:
 * - Must be in single string indicating prefix of the item typeId.
 * 
 * Details of the 'specific' Property:
 * - type: Can be a single string or an array of strings indicating the type or suffix of the item typeId.
 *   - Single string only supported "all" value.
 * - duration: A numeric value representing the duration or other time-related characteristic associated with the item.
 *   - Allowed value is a Number.
 */

const items = [
  {
    typeId: "bf:wooden_axe",
    extraDamage: 8,
    cooldown: {
      type: ["wooden", "stone"],
      duration: 1.25,
    },
  },
  {
    typeId: "bf:wooden_sword",
    extraDamage: 5,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "bf:stone_axe",
    extraDamage: 10,
    cooldown: {
      type: ["wooden", "stone"],
      duration: 1.25,
    },
  },
  {
    typeId: "bf:stone_sword",
    extraDamage: 6,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "bf:iron_axe",
    extraDamage: 10,
    cooldown: {
      type: ["iron"],
      duration: 1.1,
    },
  },
  {
    typeId: "bf:iron_sword",
    extraDamage: 7,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "bf:golden_axe",
    extraDamage: 8,
    cooldown: {
      type: ["diamond", "gold", "netherite"],
      duration: 1,
    },
  },
  {
    typeId: "bf:golden_sword",
    extraDamage: 5,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "bf:diamond_axe",
    extraDamage: 10,
    cooldown: {
      type: ["diamond", "gold", "netherite"],
      duration: 1,
    },
  },
  {
    typeId: "bf:diamond_sword",
    extraDamage: 8,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "bf:netherite_axe",
    extraDamage: 11,
    cooldown: {
      type: ["diamond", "gold", "netherite"],
      duration: 1,
    },
  },
  {
    typeId: "bf:netherite_sword",
    extraDamage: 9,
    cooldown: {
      type: "all",
      duration: 0.5,
    },
  },
  {
    typeId: "minecraft:mace",
    extraDamage: 1,
    cooldown: {
      type: "all",
      duration: 2.2,
    },
  }
];


export {
    items
}