import { world, system, ItemStack, EquipmentSlot, EnchantmentType } from '@minecraft/server';

// Define the items to detect and the items to swap with, including visual stats
const itemSwapRules = [
    { fromItem: "minecraft:wooden_axe", toItem: "bf:wooden_axe", extraDamage: 7, attackSpeed: 1.25 },
    { fromItem: "minecraft:wooden_sword", toItem: "bf:wooden_sword", extraDamage: 4, attackSpeed: 0.6 },
    { fromItem: "minecraft:stone_axe", toItem: "bf:stone_axe", extraDamage: 9, attackSpeed: 1.1 },
    { fromItem: "minecraft:stone_sword", toItem: "bf:stone_sword", extraDamage: 5, attackSpeed: 0.6 },
    { fromItem: "minecraft:iron_axe", toItem: "bf:iron_axe", extraDamage: 9, attackSpeed: 1.1 },
    { fromItem: "minecraft:iron_sword", toItem: "bf:iron_sword", extraDamage: 6, attackSpeed: 0.6 },
    { fromItem: "minecraft:golden_axe", toItem: "bf:golden_axe", extraDamage: 7, attackSpeed: 1 },
    { fromItem: "minecraft:golden_sword", toItem: "bf:golden_sword", extraDamage: 4, attackSpeed: 0.6 },
    { fromItem: "minecraft:diamond_axe", toItem: "bf:diamond_axe", extraDamage: 9, attackSpeed: 1.1 },
    { fromItem: "minecraft:diamond_sword", toItem: "bf:diamond_sword", extraDamage: 7, attackSpeed: 0.6 },
    { fromItem: "minecraft:netherite_axe", toItem: "bf:netherite_axe", extraDamage: 10, attackSpeed: 1 },
    { fromItem: "minecraft:netherite_sword", toItem: "bf:netherite_sword", extraDamage: 8, attackSpeed: 0.6 }
];

// Function to set lore with damage, attack speed, and sharpness damage
function setLore(item, damage, attackSpeed) {
    const lore = [
        `§r§7When in Main Hand:`,
        `§r§2 ${damage} Attack Damage`, // Include extra sharpness damage
        `§r§2 ${attackSpeed} Attack Speed`
    ];
    item.setLore(lore);
}
// Function to copy enchantments from one item to another
function copyEnchantments(fromItem, toItem) {
    const enchants = fromItem.getComponent("enchantable").getEnchantments();
    const newEnchant = toItem.getComponent("enchantable");

    enchants.forEach(enchant => {
        newEnchant.addEnchantment({
            level: enchant.level,
            type: enchant.type,
        });
    });
}

// Function to copy durability from one item to another
function copyDurability(fromItem, toItem) {
    const fromDurability = fromItem.getComponent("durability");
    const toDurability = toItem.getComponent("durability");

    if (fromDurability && toDurability) {
        toDurability.damage = fromDurability.damage;
    }
}

// Function to swap items in a player's inventory and apply the swap rules
function swapItems(player) {
    const inventory = player.getComponent("inventory").container;

    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);

        // If an item exists in this slot
        if (item) {
            // Check each rule for swapping
            itemSwapRules.forEach(rule => {
                if (rule.fromItem === item.typeId) {
                    // Remove the current item from the inventory
                    inventory.setItem(i, null);

                    // Create the new item to replace with
                    const newItem = new ItemStack(rule.toItem, item.amount);

                    // Copy enchantments and durability from the original item
                    copyEnchantments(item, newItem);
                    copyDurability(item, newItem);


                    // Set lore with damage, attack speed, and sharpness damage
                    setLore(newItem, rule.extraDamage, rule.attackSpeed);

                    // Add the new item back to the inventory
                    inventory.setItem(i, newItem);
                   
                }
            });
        }
    }
}

system.runInterval(() => {
    const players = world.getPlayers();

    if (players.length === 0) {
        return; // No players online, skip the rest of the logic
    }

    for (const player of players) {
        try {
            swapItems(player);
        } catch (error) {
            console.error(`Error handling player ${player.name}: ${error}`);
        }
    }
}, 1); // Every 10 seconds
