import { world, system, TicksPerSecond } from '@minecraft/server';
import { ModalFormData } from "@minecraft/server-ui";

let dynamicProperties = [
    { id: "satdis", name: "SATURATION DISPLAY", value: true },
    { id: "sat", name: "JAVA SATURATION", value: true },
    { id: "web", name: "JAVA WEBS", value: true },
    { id: "old_web", name: "OLD WEBS", value: false },
    { id: "door", name: "JAVA DOORS", value: true },
    { id: "swap", name: "TOTEM SWAP", value: true },
	{ id: "dupe", name: "STRING DUPER", value: false },
	{ id: "java", name: "JAVA KB HIT", value: false },
  { id: "death", name: "DEATH MSG", value: true }
];

function dp(player, { id }) {
    const worldValue = world.getDynamicProperty(id);
    if (worldValue === undefined) {
        const localValue = dynamicProperties.find(property => property.id === id);
        return localValue ? localValue.value : false;
    }
    return worldValue;
}

function showItemForm(player) {
    const form = new ModalFormData().title("BLURRYS PVP CONFIG");
    dynamicProperties.forEach(property => {
        form.toggle({ translate: property.name }, dp(player, { id: property.id }) ?? false);
    });
    form.show(player).then((response) => {
        if (response.canceled) {
            player.sendMessage("Configuration canceled.");
            return;
        }
        dynamicProperties.forEach((property, index) => {
            const isChecked = response.formValues[index];
            property.value = isChecked;
            world.setDynamicProperty(property.id, isChecked);
        });
        player.sendMessage(`Config Saved`);
    });
}

function gethealth(entity) {
    const healthEffect = entity.getEffect("minecraft:health_boost");
    if (!healthEffect) {return 0;}
    const amplifier = healthEffect.amplifier;
    const health = 4 * (amplifier + 1);
    return health;
}

system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { id, sourceEntity } = event;
    if (id === "bf:config") {
        showItemForm(sourceEntity)
	}
});

world.afterEvents.itemCompleteUse.subscribe(event => {
  try {
    let player = event.source;
    let item = event.itemStack;

    if (item.typeId === "bf:golden_apple") {
      player.runCommandAsync("effect @s absorption 0 0");
      player.runCommandAsync("effect @s absorption 120 0 true");
      player.addTag("gapple")
      if (player && player.isValid()) {
        system.runInterval(() => {
          player.removeTag("gapple")
        }, );
      }
    }
  } catch (error) {
  }
});

const foods = [
  { item: "minecraft:apple", duration: 2.4 },
  { item: "minecraft:baked_potato", duration: 6 },
  { item: "minecraft:beetroot", duration: 1 },
  { item: "minecraft:beetroot_soup", duration: 7.2 },
  { item: "minecraft:bread", duration: 6 },
  { item: "minecraft:cake", duration: 0.4 },
  { item: "minecraft:carrot", duration: 3.6 },
  { item: "minecraft:chorus_fruit", duration: 2.4 },
  { item: "minecraft:cooked_chicken", duration: 7.2 },
  { item: "minecraft:cooked_cod", duration: 6 },
  { item: "minecraft:muttonCooked", duration: 9.6 },
  { item: "minecraft:cooked_porkchop", duration: 12.8 },
  { item: "minecraft:cooked_rabbit", duration: 6 },
  { item: "minecraft:cooked_salmon", duration: 9.6 },
  { item: "minecraft:cookie", duration: 0.4 },
  { item: "minecraft:dried_kelp", duration: 0.2 },
  { item: "minecraft:enchanted_golden_apple", duration: 9.6 },
  { item: "bf:golden_apple", duration: 9.6 },
  { item: "minecraft:glow_berries", duration: 0.4 },
  { item: "minecraft:golden_carrot", duration: 14.4 },
  { item: "minecraft:honey_bottle", duration: 1.2 },
  { item: "minecraft:melon_slice", duration: 1.2 },
  { item: "minecraft:mushroom_stew", duration: 7.2 },
  { item: "minecraft:poisonous_potato", duration: 1.2 },
  { item: "minecraft:potato", duration: 0.6 },
  { item: "minecraft:pufferfish", duration: 0.2 },
  { item: "minecraft:pumpkin_pie", duration: 4.8 },
  { item: "minecraft:rabbit_stew", duration: 12 },
  { item: "minecraft:beef", duration: 1.8 },
  { item: "minecraft:chicken", duration: 1.2 },
  { item: "minecraft:cod", duration: 0.4 },
  { item: "minecraft:muttonRaw", duration: 1.2 },
  { item: "minecraft:porkchop", duration: 1.8 },
  { item: "minecraft:rabbit", duration: 1.8 },
  { item: "minecraft:salmon", duration: 0.4 },
  { item: "minecraft:rotten_flesh", duration: 0.8 },
  { item: "minecraft:spider_eye", duration: 3.2 },
  { item: "minecraft:cooked_beef", duration: 12.8 },
  { item: "minecraft:suspicious_stew", duration: 7.2 },
  { item: "minecraft:sweet_berries", duration: 1.2 },
  { item: "minecraft:tropical_fish", duration: 0.2 }
];

function applySaturation(player, duration) {
  let saturationTime = player.getDynamicProperty("saturationTime") || 0;
  saturationTime += duration;
  if (saturationTime > 20) {
    saturationTime = 20;
  }
  player.setDynamicProperty("saturationTime", saturationTime);
  player.addTag("dave:onSaturationEffect");
}

world.afterEvents.itemCompleteUse.subscribe(({ itemStack, source }) => {
  foods.forEach(food => {
    if (itemStack.typeId === food.item) {
      applySaturation(source, food.duration);
    }
  });
});

system.runInterval(() => {
  world.getAllPlayers().forEach(player => {
	const isSaturationOn = world.getDynamicProperty("sat");
    if (player.hasTag("dave:onSaturationEffect") && isSaturationOn) {
      let saturationTime = player.getDynamicProperty("saturationTime") || 0;
		
      if (saturationTime > 0) {
        let currentHealth = player.getComponent("minecraft:health").currentValue;
        if (currentHealth < 20 + gethealth(player)) {
		  saturationTime -= 1;
		  player.setDynamicProperty("saturationTime", saturationTime);
          if (player.hasTag("gapple")) {
            player.addEffect("regeneration", 1 * TicksPerSecond, {
              amplifier: 5,
              showParticles: false,
            });
          } else {
            player.addEffect("regeneration", 1 * TicksPerSecond, {
              amplifier: 3,
              showParticles: false,
            });
          }
          player.addEffect("hunger", 1 * TicksPerSecond, {
            amplifier: 110,
            showParticles: false,
          });
		  system.runTimeout(() => {
        player.runCommandAsync(`effect @s clear regeneration`);
				player.runCommandAsync(`effect @s clear hunger`);
            }, 5);
            if (saturationTime < 0) {
              saturationTime = 0;
            }
        }
      } else {
        player.setDynamicProperty("saturationTime", 0);
        player.removeTag("dave:onSaturationEffect");
      }
	}
  });
}, 10);

system.runInterval(() => {
  world.getAllPlayers().forEach(player => {
    let sat = 'notification.';
    let saturationTime = player.getDynamicProperty("saturationTime") || 0;
  const isSaturationDisplayOn = world.getDynamicProperty("satdis");
    if (isSaturationDisplayOn && !player.matches({gameMode: 'creative'}) && !player.matches({gameMode: 'spectator'})) {
      if (saturationTime === 0) {
        sat = `notification.`;
      }
      if (saturationTime >= 1 && saturationTime <= 2) {
        sat = `notification.`;
      }
      if (saturationTime >= 2 && saturationTime <= 3) {
        sat = `notification.`;
      }
      if (saturationTime >= 3 && saturationTime <= 4) {
        sat = `notification.`;
      }
      if (saturationTime >= 4 && saturationTime <= 5) {
        sat = `notification.`;
      }
      if (saturationTime >= 5 && saturationTime <= 6) {
        sat = `notification.`;
      }
      if (saturationTime >= 6 && saturationTime <= 7) {
        sat = `notification.`;
      }
      if (saturationTime >= 7 && saturationTime <= 8) {
        sat = `notification.`;
      }
      if (saturationTime >= 8 && saturationTime <= 9) {
        sat = `notification.`;
      }
      if (saturationTime >= 9 && saturationTime <= 10) {
        sat = `notification.`;
      }
      if (saturationTime >= 10 && saturationTime <= 11) {
        sat = `notification.`;
      }
      if (saturationTime >= 11 && saturationTime <= 12) {
        sat = `notification.`;
      }
      if (saturationTime >= 12 && saturationTime <= 13) {
        sat = `notification.`;
      }
      if (saturationTime >= 13 && saturationTime <= 14) {
        sat = `notification.`;
      }
      if (saturationTime >= 14 && saturationTime <= 15) {
        sat = `notification.`;
      }
      if (saturationTime >= 15 && saturationTime <= 16) {
        sat = `notification.`;
      }
      if (saturationTime >= 16 && saturationTime <= 17) {
        sat = `notification.`;
      }
      if (saturationTime >= 17 && saturationTime <= 18) {
        sat = `notification.`;
      }
      if (saturationTime >= 18 && saturationTime <= 19) {
        sat = `notification.`;
      }
      if (saturationTime >= 19 && saturationTime <= 20) {
        sat = `notification.`;
      }
      if (saturationTime === 20) {
        sat = `notification.`;
      }
      player.onScreenDisplay.setActionBar(sat);
    }
  });
}, 2);

world.afterEvents.worldInitialize.subscribe(() => {
    system.runTimeout(() => {
		if (world.getDynamicProperty("satdis") === undefined) {
			world.setDynamicProperty("satdis", true);
		}
		if (world.getDynamicProperty("sat") === undefined) {
			world.setDynamicProperty("sat", true);
		}
		if (world.getDynamicProperty("web") === undefined) {
			world.setDynamicProperty("web", true);
		}
		if (world.getDynamicProperty("door") === undefined) {
			world.setDynamicProperty("door", true);
		}
		if (world.getDynamicProperty("swap") === undefined) {
			world.setDynamicProperty("swap", true);
		}
		if (world.getDynamicProperty("old_web") === undefined) {
			world.setDynamicProperty("old_web", false);
		}
		if (world.getDynamicProperty("dupe") === undefined) {
			world.setDynamicProperty("dupe", false);
		}
		if (world.getDynamicProperty("java") === undefined) {
			world.setDynamicProperty("java", false);
		}
    if (world.getDynamicProperty("death") === undefined) {
			world.setDynamicProperty("death", true);
		}
        world.sendMessage("§6§lUse §f/scriptevent minecraft:config §6For config");
    }, 80);
});

world.afterEvents.entitySpawn.subscribe((event) => {
  const entity = event.entity;
  if (entity.typeId === "minecraft:player") {
      entity.setDynamicProperty("saturationTime", 6);
  }
});

world.afterEvents.playerSpawn.subscribe(({ player }) => {
      player.setDynamicProperty("saturationTime", 6);
      player.addTag("dave:onSaturationEffect");
      const effect = player.getEffect("minecraft:absorption");
      if (effect) {
        player.runCommandAsync("effect @s absorption 0 0");
        player.runCommandAsync("effect @s absorption 120 0 true");
      }
});

system.runInterval(() => {
  world.getAllPlayers().forEach(player => {
    let sat = player.getDynamicProperty("saturationTime") ?? 6; // Ensure default value if undefined
    sat = Math.max(0, sat - 1); // Prevent negative values
    player.setDynamicProperty("saturationTime", sat);
  });
}, 400);
