import { world, system, ItemStack } from "@minecraft/server";

// Food swap data mapped for quick access
const foodSwapMap = new Map([
  ["minecraft:enchanted_golden_apple", { newItem: "bf:enchanted_golden_apple", duration: 9.6 }],
  ["minecraft:golden_apple", { newItem: "bf:golden_apple", duration: 9.6 }],
]);

// Function to set lore for food items

// Function to swap food items in a player's inventory
function swapFood(player) {
  const inventory = player.getComponent("inventory").container;

  for (let i = 0; i < inventory.size; i++) {
    const item = inventory.getItem(i);
    if (!item) continue;

    const swapData = foodSwapMap.get(item.typeId);
    if (swapData) {
      const { newItem, duration } = swapData;

      const newItemStack = new ItemStack(newItem, item.amount);

      inventory.setItem(i, newItemStack);
    }
  }
}

// Run the swap function for all players periodically
system.runInterval(() => {
  world.getPlayers().forEach(player => {
    try {
      swapFood(player);
    } catch (error) {
      console.error(`Error swapping food for ${player.name}: ${error}`);
    }
  });
}, 20); // Runs every second
